<?php

namespace Drupal\spreadsheets\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Entity\File;
use Google_Client;
use Google_Service_Sheets;

// Use Drupal\Core\Routing\TrustedRedirectResponse;.
/**
 * Class SpreadsheetsCredentials.
 */
class SpreadsheetsCredentials extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'spreadsheets.spreadsheetscredentials',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'spreadsheets_credentials';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    if (!(\Drupal::hasService('stream_wrapper.private'))) {
      $form['warning'] = [
        '#markup' => 'Please <a href="https://www.drupal.org/docs/8/modules/skilling/installation/set-up-a-private-file-path" target="_blank">setup the private file system </a>
      to use this module',
      ];
      return $form;
    }

    $config = $this->config('spreadsheets.spreadsheetscredentials');

    $form['credentials_file'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('credentials'),
      '#description' => $this->t('credentials file'),
      '#default_value' => $config->get('credentials_file'),
      '#upload_location' => 'private://',
      '#upload_validators' => [
        'file_validate_extensions' => ['json'],
      ],
    ];

    if (!empty($config->get('credentials_file')[0])) {
      $fid = $config->get('credentials_file')[0];
      $file = File::load($fid);
      $credentialsPath = \Drupal::service('file_system')->realpath('private://' . $file->getFilename());
      $client = new Google_Client();
      $client->setApplicationName('Google Sheets API PHP Quickstart');
      $client->setScopes(Google_Service_Sheets::SPREADSHEETS_READONLY);
      $client->setAuthConfig($credentialsPath);
      $client->setAccessType('offline');
      $authUrl = $client->createAuthUrl();

      $form['token'] = [
        '#type' => 'textfield',
        '#title' => $this
          ->t('Token'),
        '#default_value' => $config->get('token'),
        '#size' => 256,
        '#maxlength' => 256,
      ];

      $form['token_link'] = [
        '#markup' => '<a href="' . $authUrl . '" target="_blank">get new token</a>',
      ];

      $tokenPath = \Drupal::service('file_system')->realpath('private://token.json');
      if (!file_exists($tokenPath) && !empty($config->get('token'))) {
        $authCode = trim($config->get('token'));
        $accessToken = $client->fetchAccessTokenWithAuthCode($authCode);
        // Check to see if there was an error.
        if (array_key_exists('error', $accessToken)) {
          $messenger = \Drupal::messenger();
          $messenger->addError('There was an error creating your access token' . implode(', ', $accessToken));
        }
        else {
          // Store the credentials to disk.
          if (!file_exists(dirname($tokenPath))) {
            mkdir(dirname($tokenPath), 0700, TRUE);
          }
          file_put_contents($tokenPath, json_encode($accessToken));
        }
      }
    }

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $form_file = $form_state->getValue('credentials_file');
    if (isset($form_file[0]) && !empty($form_file[0])) {
      $file = File::load($form_file[0]);
      $file->setPermanent();
      $file->save();
    }

    $this->config('spreadsheets.spreadsheetscredentials')
      ->set('credentials_file', $form_state->getValue('credentials_file'))
      ->set('token', $form_state->getValue('token'))
      ->save();
  }

}
